function [ boolean, float_imeis ] = fetch_process_mails( p_config, p_metadata )
%FETCH_MAILS Fetches the mails of the floats in p_metadata
%   Fetches mails of the given floats in p_metadata. For this, javax.mail
%   package is being used.
%
%   Parameters:
%   p_config:           Configuration structure, must contain the
%                       following values:
%                       -username {string}
%                       -password {string}
%                       -port {integer}
%                       -ssl {0 or 1}
%                       -server {string}
%                       -imap_timeout {integer}
%                       -imaps_connection_timeout {integer}
%                       -storage_directory {string}
%                       -folder_to_fetch {string}
%                       -sort_by_imei {0 or 1}
%                       -separate_sbd_and_txt {0 or 1}
%                       -mark_read_after_fetch {0 or 1}
%                       -fetch_unread_only {0 or 1}
%                       -sender_address {string}
%                       -use_sender_address {0 or 1}
%
%   Returns:
%   boolean:            True, if successful.
%   float_imeis         List with length of p_metadata. Contains either 0
%                       or the IMEI number of the float if there are new
%                       mails that have been fetched.

%% Initialize return variables
boolean = false;

%% Parameter check
if (~isstruct(p_config) || (~isstruct(p_metadata) && ~iscell(p_metadata)))
    warning([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
end

%% Setup some alias
fetch_unread_only = p_config.MAIL.fetch_unread_only;
mark_read_after_fetch = p_config.MAIL.mark_read_after_fetch;
sort_by_imei = p_config.MAIL.sort_by_imei;
storage_directory = p_config.MAIL.storage_directory;
separate_sbd_and_txt = p_config.MAIL.separate_sbd_and_txt;

%% Setup folder
if (~isdir(storage_directory))
    mkdir(storage_directory);
end

%% Connect to mail server
imapstore = nemo.mail.connect(p_config);
if (~imapstore.isConnected()) % warning is already given in connect()
    return;
end

%% Create search and flag term
% there are difficulties accessing static class members of an inner java
% class. This solution has been found at:
% http://undocumentedmatlab.com/blog/accessing-internal-java-class-members
if (fetch_unread_only)
    search_flag = javax.mail.Flags(); % create search flag
    tmp = search_flag.getClass.getDeclaredClasses;
    seen_flag = tmp(1).getField('SEEN').get(1);
    clear tmp;
    search_flag.add(seen_flag);
    flag_term = javax.mail.search.FlagTerm(search_flag, false);
else
    flag_term = false;
end

if (~isfield(p_config.MAIL, 'sender_address') || ~isfield(p_config.MAIL, 'use_sender_address'))
    warning([mfilename ': Config does not include a MAIL.sender_address or MAIL.use_sender_address field!']);
    return;
elseif (p_config.MAIL.use_sender_address)
    java_sender_address = java.lang.String(p_config.MAIL.sender_address);
    sender_term = javax.mail.search.FromStringTerm(java_sender_address);
else
    sender_term = false;
end



%% Fetch all mails and save them to disk

switch class(p_metadata)
    case 'struct'
        % only one float has to be processed
        [boolean, float_imeis] = process_float(p_metadata);
    case 'cell'
        [booleans, float_imeis] = cellfun(@process_float, p_metadata);
        if (~all(booleans))
            warning([mfilname ': Something went wrong during message download! Please contact developer or system administrator!']);
        else
            boolean = true;
        end
end

clear imapstore;


    function [boolean, float_imei] = process_float(p_float_metadata)
        imei_int = p_float_metadata.GENERAL_INFO.IMEI;
        imei_str = int2str(imei_int);
        float_imei = 0; % will be overridden if there are messages to process
        %% Check for directories
        if (sort_by_imei)
            float_storage_directory = [storage_directory imei_str p_config.GENERAL.folder_separator];
        else
            float_storage_directory = storage_directory;
        end
        % check if folders exists
        if (~isdir(float_storage_directory))
            mkdir(float_storage_directory);
        end
        if (separate_sbd_and_txt)
            float_txt_directory = [float_storage_directory 'txt' p_config.GENERAL.folder_separator];
            float_sbd_directory = [float_storage_directory 'sbd' p_config.GENERAL.folder_separator];
            float_incomplete_directory = [float_storage_directory 'incomplete' p_config.GENERAL.folder_separator];
            if (~isdir(float_sbd_directory))
                mkdir(float_sbd_directory);
            end
            if (~isdir(float_txt_directory))
                mkdir(float_txt_directory);
            end
            if (~isdir(float_incomplete_directory))
                mkdir(float_incomplete_directory);
            end
        else
            float_sbd_directory = float_storage_directory;
            float_txt_directory = float_sbd_directory;
        end
        
        %% Create term for message filtering
        subject_term = javax.mail.search.SubjectTerm(imei_str);
        if isa(flag_term, 'javax.mail.search.FlagTerm') ...
                && isa(sender_term, 'javax.mail.search.FromStringTerm')
            tmp_comb = javax.mail.search.AndTerm(subject_term, flag_term);
            combined_terms = javax.mail.search.AndTerm(tmp_comb, sender_term);
        elseif isa(flag_term, 'javax.mail.search.FlagTerm')
            combined_terms = javax.mail.search.AndTerm(flag_term, subject_term);
        elseif isa(sender_term, 'javax.mail.search.FromStringTerm')
            combined_terms = javax.mail.search.AndTerm(sender_term, subject_term);
        else
            combined_terms = subject_term;
        end
        
        %% Get mail that fit the terms
        [messages, folder] = nemo.mail.fetch_messages(imapstore, ...
            p_config.MAIL.folder_to_fetch, ...
            combined_terms, ...
            ~mark_read_after_fetch ...
        );
    
        %% Process all messages
        for i = 1:messages.length
            [from, ~, inline, attachments] = nemo.mail.extract(messages(i));
            if (p_config.MAIL.use_sender_address && ~strcmp(from, p_config.MAIL.sender_address))
                disp([mfilename ': Filter fetched the wrong address: ' from{1} '! Skipping message...']);
                continue;
            end
            
            if (~isempty(attachments))
                % store the float serial
                float_imei = imei_int;
                filename = char(attachments{1});
                [~, file, ~] = fileparts(filename);
                % export inline as well as attachment
                nemo.mail.export_attachment(attachments{2}, [float_sbd_directory filename]);
                % save inline
                fid = fopen([float_txt_directory file '.txt'], 'wt');
                fprintf(fid, inline);
                fclose(fid);
            else % mail has been received but no attachment
                % export only inline to incomplete folder
                processed_inline = nemo.mail.process_inline(inline);
                fid = fopen([float_incomplete_directory int2str(processed_inline.MOMSN) '.txt'], 'wt');
                fprintf(fid, inline);
                fclose(fid);
            end
            clear from subject inline attachments filename;
        end
        
        if (mark_read_after_fetch && messages.length > 0)
            % mark processed as read
            flag = javax.mail.Flags(); % create search flag
            tmp_class = flag.getClass.getDeclaredClasses;
            flag_seen = tmp_class(1).getField('SEEN').get(1);
            clear tmp_class;
            flag.add(flag_seen);
            folder.setFlags(messages, flag, true);
            clear flag
        end
        clear messages combined_terms subject_term messages folder;
        
        boolean = true;
        
    end


end

