function [ boolean ] = timing_information( p_nemo_data, p_config )
%TIMING_INFORMATION Plots clock details.
%
%   Returns:
%   boolean: False if something went wrong during the plotting process.
%
%   Parameters:
%   p_nemo_data:      Struct, containing all float data (load with nemo.load.data()).
%   p_config:         Global configuration file. Load using nemo.load.config().

%% Initialize return variable
boolean = false;

%% Parameter check
parameter_error = false;

if (~iscell(p_nemo_data) || ~isstruct(p_config))
    parameter_error = true;
end

if (parameter_error) % display error message
    disp([ mfilename ': Wrong parameter! Please check your input parameters carefully!']);
    return;
else % do some cleanup
    clear parameter_error;
end

%% Setup directories
directory = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TIMING_INFORMATION.storage_directory_image);
directory_fig = fullfile(p_config.GENERAL.storage_directory_plots, p_config.PLOT_TIMING_INFORMATION.storage_directory_fig);
image_type = p_config.PLOT_TIMING_INFORMATION.image_type;
if ~exist(directory, 'dir')
    system(['mkdir ' directory]);
end
if ~exist(directory_fig, 'dir')
    mkdir(directory_fig);
end

%% Process every float

% create dummy legend item to make the boxes be all of the same size

float_count = length(p_nemo_data);

for o_float = 1:float_count
    
    
    gps_profile_data = struct();
    all_profile_data = struct();
    all_profile_data.profiles = p_nemo_data{o_float};
    all_profile_data.best_guess_position_src = nemo.float.collect(p_nemo_data{o_float}, 'SECOND_ORDER_INFORMATION', 'best_guess_position_src');
    all_profile_data.best_guess_datevec = nemo.float.collect(p_nemo_data{o_float}, 'SECOND_ORDER_INFORMATION', 'best_guess_datevec');
    
    gps_profile_data.profiles = {};
    for i = 1:length(all_profile_data.best_guess_position_src)
        if (strcmp(all_profile_data.best_guess_position_src{i}, 'gps'))
            gps_profile_data.profiles{end + 1} = p_nemo_data{o_float}{i};
        end
    end
%     
%     if isempty(gps_profile_data.profiles)
%         continue
%     end
    
    gps_profile_data.best_guess_datevec = nemo.float.collect(gps_profile_data.profiles, 'SECOND_ORDER_INFORMATION', 'best_guess_datevec');
    
    all_profile_data.start_date = p_nemo_data{o_float}{end}.OVERALL_MISSION_INFORMATION.start_date;
    all_profile_data.start_time = p_nemo_data{o_float}{end}.OVERALL_MISSION_INFORMATION.start_time;
    gps_profile_data.surface_gps_data = nemo.float.collect(gps_profile_data.profiles, 'SURFACE_GPS_DATA');
    
    plot_data = struct();
    % required data for plot 1
    
    % SURFACESTART - GPSFix
    if ~isempty(gps_profile_data.profiles)
        plot_data.surfaceStart_GPS = struct();
        gps_profile_data.xmit_surface_start_datevec = nemo.float.collect(gps_profile_data.profiles, 'xmitDatevec', 'xmit_surface_start_time');
        plot_data.surfaceStart_GPS.y = ( ...
            datenum(gps_profile_data.xmit_surface_start_datevec) - ...
            datenum(gps_profile_data.surface_gps_data(:, 8:13)) ...
            ) * 86400;
        plot_data.surfaceStart_GPS.x = datenum(gps_profile_data.best_guess_datevec);
    end
    
    % RTC0015 - GPSFix
    if ~(isempty(gps_profile_data.profiles))
        gps_profile_data.profile_data = nemo.float.collect(gps_profile_data.profiles, 'PROFILE_DATA');
        plot_data.RTC0015_GPSFix = struct();
        % get values near to 15m depth
        [~, indices] = cellfun(@(x) find_nearest_value_to(x(:, 7), 15), gps_profile_data.profile_data, 'UniformOutput', false);
        time_15m = cellfun(@(x,y) x(y, 2:4), gps_profile_data.profile_data, indices, 'UniformOutput', false);
        time_15m = cell2mat(time_15m');
        gps_profile_data.RTC0015_datevec = gps_profile_data.surface_gps_data(:, 8:13);
        gps_profile_data.RTC0015_datevec(:, 4:6) = time_15m;
        tmp_subtract_one_day = (gps_profile_data.surface_gps_data(:, 11) >= 21 & gps_profile_data.surface_gps_data(:, 11) <= 23) ...
            & (time_15m(:, 1) >= 0 & time_15m(:, 1) <= 3);
        tmp_add_one_day = (gps_profile_data.surface_gps_data(:, 11) >= 0 & gps_profile_data.surface_gps_data(:, 11) <= 3) ...
            & (time_15m(:, 1) >= 21 & time_15m(:, 1) <= 23);
        
        if (any(tmp_subtract_one_day))
            % GPS is more trustworthy, so reduce the RTC date by 1
            tmp_indices = find(tmp_subtract_one_day);
            for i = 1:length(tmp_indices)
                gps_profile_data.RTC0015_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([gps_profile_data.RTC0015_datevec(tmp_indices(i), 1:3), time_15m(tmp_indices(i), :)]), -1, 'day'));
            end
        end
        if (any(tmp_add_one_day))
            % same here, but now add 1 day
            tmp_indices = find(tmp_add_one_day);
            for i = 1:length(tmp_indices)
                gps_profile_data.RTC0015_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([gps_profile_data.RTC0015_datevec(tmp_indices(i), 1:3), time_15m(tmp_indices(i), :)]), +1, 'day'));
            end
        end
        clear tmp_indices tmp_subtract_one_day tmp_add_one_day
        
        plot_data.RTC0015_GPSFix.y = ( ...
            datenum(gps_profile_data.RTC0015_datevec) - ...
            datenum(gps_profile_data.surface_gps_data(:, 8:13)) ...
            ) * 86400;
        plot_data.RTC0015_GPSFix.x = datenum(gps_profile_data.best_guess_datevec);
    end
    
    
    % SURFACESTART - IRIDIUM
    if ~isempty(gps_profile_data.profiles)
    plot_data.surfaceStart_Iridium = struct();
    gps_profile_data.iridium_data = nemo.float.collect(gps_profile_data.profiles, 'IRIDIUM_DATA');
    plot_data.surfaceStart_Iridium.y = ( ...
        datenum(gps_profile_data.xmit_surface_start_datevec) - ...
        datenum(gps_profile_data.iridium_data(:, 4:9)) ...
        ) * 86400;
    plot_data.surfaceStart_Iridium.x = datenum(gps_profile_data.best_guess_datevec);
    end
    
    % ASCENTSTART - RTC2000
    plot_data.ascentStart_RTC2000 = struct();
    all_profile_data.xmit_ascent_start_datevec = nemo.float.collect(all_profile_data.profiles, 'xmitDatevec', 'xmit_ascent_start_time');
    all_profile_data.profile_data = nemo.float.collect(all_profile_data.profiles, 'PROFILE_DATA');
    all_profile_data.profile_data_2000 = cellfun(@(x) x(end, 2:4), all_profile_data.profile_data, 'UniformOutput', false);
    all_profile_data.profile_data_2000 = cell2mat(all_profile_data.profile_data_2000');
    all_profile_data.profile_data_2000_datevec = datevec(datetime([all_profile_data.xmit_ascent_start_datevec(:, 1:3), all_profile_data.profile_data_2000]));
   
    tmp_subtract_one_day = (all_profile_data.profile_data_2000(:, 1) >= 21 & all_profile_data.profile_data_2000(:, 1) <= 23) ...
        & (all_profile_data.xmit_ascent_start_datevec(:, 4) >= 0 & all_profile_data.xmit_ascent_start_datevec(:, 4) <= 3);
    tmp_add_one_day = (all_profile_data.profile_data_2000(:, 1) >= 0 & all_profile_data.profile_data_2000(:, 1) <= 3) ...
        & (all_profile_data.xmit_ascent_start_datevec(:, 4) >= 21 & all_profile_data.xmit_ascent_start_datevec(:, 4) <= 23);
    
    if (any(tmp_subtract_one_day))
        % RTC is more trustworthy, so reduce the software date by 1
        tmp_indices = find(tmp_subtract_one_day);
        for i = 1:length(tmp_indices)
            all_profile_data.profile_data_2000_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([all_profile_data.xmit_ascent_start_datevec(tmp_indices(i), 1:3), all_profile_data.profile_data_2000(tmp_indices(i), :)]), -1, 'day'));
        end
    end
    if (any(tmp_add_one_day))
        % same here, but now add 1 day
        tmp_indices = find(tmp_add_one_day);
        for i = 1:length(tmp_indices)
            all_profile_data.profile_data_2000_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([all_profile_data.xmit_ascent_start_datevec(tmp_indices(i), 1:3), all_profile_data.profile_data_2000(tmp_indices(i), :)]), +1, 'day'));
        end
    end
    clear tmp_indices tmp_subtract_one_day tmp_add_one_day
    plot_data.ascentStart_RTC2000.y = ( ...
        datenum(all_profile_data.xmit_ascent_start_datevec) - ...
        datenum(all_profile_data.profile_data_2000_datevec) ...
        ) * 86400;
    plot_data.ascentStart_RTC2000.x = datenum(all_profile_data.best_guess_datevec);
    
    % SURFACESTART - RTC0000
    plot_data.surfaceStart_RTC0000 = struct();
    all_profile_data.xmit_surface_start_datevec = nemo.float.collect(all_profile_data.profiles, 'xmitDatevec', 'xmit_surface_start_time');
    all_profile_data.profile_data_0000 = cellfun(@(x) x(1, 2:4), all_profile_data.profile_data, 'UniformOutput', false);
    all_profile_data.profile_data_0000 = cell2mat(all_profile_data.profile_data_0000');
    all_profile_data.profile_data_0000_datevec = datevec(datetime([all_profile_data.xmit_surface_start_datevec(:, 1:3), all_profile_data.profile_data_0000]));
        
    tmp_subtract_one_day = (all_profile_data.profile_data_0000(:, 1) >= 21 & all_profile_data.profile_data_0000(:, 1) <= 23) ...
        & (all_profile_data.xmit_surface_start_datevec(:, 4) >= 0 & all_profile_data.xmit_surface_start_datevec(:, 4) <= 3);
    tmp_add_one_day = (all_profile_data.profile_data_0000(:, 1) >= 0 & all_profile_data.profile_data_0000(:, 1) <= 3) ...
        & (all_profile_data.xmit_surface_start_datevec(:, 4) >= 21 & all_profile_data.xmit_surface_start_datevec(:, 4) <= 23);
    
    if (any(tmp_subtract_one_day))
        % RTC is more trustworthy, so reduce the software date by 1
        tmp_indices = find(tmp_subtract_one_day);
        for i = 1:length(tmp_indices)
            all_profile_data.profile_data_0000_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([all_profile_data.xmit_surface_start_datevec(tmp_indices(i), 1:3), all_profile_data.profile_data_0000(tmp_indices(i), :)]), -1, 'day'));
        end
    end
    if (any(tmp_add_one_day))
        % same here, but now add 1 day
        tmp_indices = find(tmp_add_one_day);
        for i = 1:length(tmp_indices)
            all_profile_data.profile_data_0000_datevec(tmp_indices(i), :) = datevec(addtodate(datenum([all_profile_data.xmit_surface_start_datevec(tmp_indices(i), 1:3), all_profile_data.profile_data_0000(tmp_indices(i), :)]), +1, 'day'));
        end
    end
    clear tmp_indices tmp_subtract_one_day tmp_add_one_day
    plot_data.surfaceStart_RTC0000.y = ( ...
        datenum(all_profile_data.xmit_surface_start_datevec) - ...
        datenum(all_profile_data.profile_data_0000_datevec) ...
        ) * 86400;
    plot_data.surfaceStart_RTC0000.x = datenum(all_profile_data.best_guess_datevec);
    
    
    % required data for plot 2
    % get values near to 15m depth
    clear indices
    [~, indices] = cellfun(@(x) find_nearest_value_to(x(:, 7), 15), all_profile_data.profile_data, 'UniformOutput', false);
    all_time_15m = cellfun(@(x,y) x(y, 2:4), all_profile_data.profile_data, indices, 'UniformOutput', false);
    all_time_15m = cell2mat(all_time_15m');
    
    % required data for plot 3
    % GPS - IRIDIUM
    if ~isempty(gps_profile_data.profiles)
    plot_data.GPS_Iridium = struct();
    plot_data.GPS_Iridium.y = ( ...
        datenum(gps_profile_data.surface_gps_data(:, 8:13)) - ...
        datenum(gps_profile_data.iridium_data(:, 4:9)) ...
        ) * 86400;
    plot_data.GPS_Iridium.x = datenum(gps_profile_data.best_guess_datevec);
    
    % GPS - GPSRTC
    plot_data.GPS_GPSRTC = struct();
    plot_data.GPS_GPSRTC.y = ( ...
        datenum(gps_profile_data.surface_gps_data(:, 8:13)) - ...
        datenum(gps_profile_data.surface_gps_data(:, 2:7)) ...
        ) * 86400;
    plot_data.GPS_GPSRTC.x = datenum(gps_profile_data.best_guess_datevec);
    end
    
    % get plot details
    width = p_config.PLOT_TIMING_INFORMATION.image_width;
    height = p_config.PLOT_TIMING_INFORMATION.image_height;
    paperunit = p_config.PLOT_TIMING_INFORMATION.image_paperunit;
    
    % plot collected data
    
    figure_id = figure('Visible', 'off'); % create figure
    % initialize figure
    set(figure_id,'InvertHardcopy','on');
    set(figure_id,'PaperUnits', paperunit);
    papersize = get(figure_id, 'PaperSize');
    left = (papersize(1)- width)/2;
    bottom = (papersize(2)- height)/2;
    myfiguresize = [left, bottom, width, height];
    set(figure_id,'PaperPosition', myfiguresize);
    
    clear height width left bottom papersize myfiguresize
    
    %set(gcf, 'Position', get(0,'ScreenSize'), 'color', [1 1 1]);
    sub = {};
    figure_count = 3;
    
    sub{1} = subplot(figure_count,1,1);
    hold on

    %yyaxis left
    if ~isempty(gps_profile_data.profiles)
    plot(plot_data.surfaceStart_GPS.x, plot_data.surfaceStart_GPS.y, ...
         'color', [1 0 0], ...
         'marker', 'o', ...
         'MarkerFaceColor', [1 0 0], ...
         'MarkerEdgeColor', [1 0 0], ...
         'linestyle', '-', ...
         'linewidth', 1,...
         'markersize', 5);
    plot(plot_data.surfaceStart_Iridium.x, plot_data.surfaceStart_Iridium.y, ...
         'color', [0 0.5 0], ...
         'marker', 'x', ...
         'MarkerEdgeColor', [0 0.5 0], ...
         'linestyle', '-', ...
         'linewidth', 1, ...
         'markersize', 10);
     plot(plot_data.RTC0015_GPSFix.x, plot_data.RTC0015_GPSFix.y, ...
         'color', [0 0.5 1], ...
         'marker', 'd', ...
         'MarkerEdgeColor', [0 0.5 1], ...
         'linestyle', '-', ...
         'linewidth', 1, ...
         'markersize', 10);
    end
     plot(plot_data.ascentStart_RTC2000.x, plot_data.ascentStart_RTC2000.y, ...
         'color', [1 0 1], ...
         'marker', '+', ...
         'MarkerEdgeColor', [1 0 1], ...
         'linestyle', '-', ...
         'linewidth', 1, ...
         'markersize', 10);
     plot(plot_data.surfaceStart_RTC0000.x, plot_data.surfaceStart_RTC0000.y, ...
         'color', [0 0 1], ...
         'marker', 's', ...
         'MarkerEdgeColor', [0 0 1], ...
         'linestyle', '-', ...
         'linewidth', 1, ...
         'markersize', 10);


    t = title([ 'Float ' ...
          num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
          ' @ ' ...
          datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
        ]);
    set(t, 'FontSize', 18);
    ylabel('Time differences [sec]');
    if ~isempty(gps_profile_data.profiles)
        leg1 = legend('SurfaceStart-GPSFix', 'SurfaceStart-IRIDIUM', 'RTC0015 - GPSFix', 'AscentStart-RTC2000', 'SurfaceStart-RTC0000');
    else
        leg1 = legend('AscentStart-RTC2000', 'SurfaceStart-RTC0000');
    end
    set(leg1, 'Location', 'eastoutside', 'Orientation', 'vertical');
    grid on
    box on
    hold off
    
    sub{2} = subplot(figure_count,1,2);
    hold on
    ylabel('Time of day [HH:MM]');
    
    if ~isempty(gps_profile_data.profiles)
        scatter(datenum(gps_profile_data.surface_gps_data(:, 8:13)), mod(datenum(gps_profile_data.surface_gps_data(:, 8:13)), 1), 20, ...
            'marker', 'o', ...
            'MarkerFaceColor', '1 0 0', ...
            'MarkerEdgeColor', '1 0 0');
        scatter(datenum(gps_profile_data.best_guess_datevec), mod(datenum(gps_profile_data.iridium_data(:, 4:9)), 1), 50, ...
            'marker', 's', ...
            'MarkerEdgeColor', '0 0.5 0');
        
    end
    
    scatter(datenum(all_profile_data.best_guess_datevec), mod(datenum([repmat([1 1 1970], size(all_time_15m, 1), 1), all_time_15m]), 1), 50, ...
            'marker', '^', ...
            'MarkerEdgeColor', '0 0.5 1');
    scatter(datenum(all_profile_data.best_guess_datevec), mod(datenum(all_profile_data.xmit_surface_start_datevec), 1), 50, ...
            'marker', '+', ...
            'MarkerEdgeColor', '0 0 1');

    set(gca, 'YLimMode', 'manual',...
             'YLim', [0, 1], ...
             'YTick', [0:6:24]./24);
    datetick('y','HH:mm', 'keepticks');


    if ~isempty(gps_profile_data.profiles)
        legend('GPSFix', 'IRIDIUM', 'RTC0015', 'SurfaceStart');
    else
        legend('RTC0015', 'SurfaceStart');
    end
    legend('Location', 'eastoutside', 'Orientation', 'vertical');
    box on
    grid on
    hold off
    
    sub{3} = subplot(figure_count,1,3);
    hold on
    if ~isempty(gps_profile_data.profiles)
        plot(plot_data.GPS_Iridium.x, plot_data.GPS_Iridium.y, ...
            'color', [0 0 1], ...
            'marker', '+', ...
            'MarkerEdgeColor', [0 0 1], ...
            'linestyle', '-', ...
            'linewidth', 1, ...
            'markersize', 10);
        
        plot(plot_data.GPS_GPSRTC.x, plot_data.GPS_GPSRTC.y, ...
            'color', [0 1 1], ...
            'marker', '.', ...
            'MarkerEdgeColor', [0 1 1], ...
            'linestyle', '-', ...
            'linewidth', 1, ...
            'markersize', 10);
        
    else
        dim = [0.5 0 .3 .3];
        str = 'No data!';
        a = annotation('textbox', dim, 'String', str, 'FitBoxToText', 'on');
        a.FontSize = 16;
        clear dim str a
    end
    
    ylabel('Time differences [sec]');
    legend('GPSFix-IRIDIUM', 'GPSFix-GPSRTC');
    legend('Location', 'eastoutside', 'Orientation', 'vertical');
    grid on
    box on
    hold off
    
    sub1_pos = get(sub{1}, 'Position');
    sub2_pos = get(sub{2}, 'Position');
    sub3_pos = get(sub{3}, 'Position');
%     leg1_pos = get(leg1, 'Position');
    set(sub{1}, 'Position', [sub1_pos(1:2) 0.6652 sub1_pos(4)]);
    set(sub{2}, 'Position', [sub2_pos(1:2) 0.6652 sub2_pos(4)]);
    set(sub{3}, 'Position', [sub3_pos(1:2) 0.6652 sub3_pos(4)]);
    
%     % align all legends
%     leg1_pos = get(hleg1, 'position');
%     leg2_pos = get(hleg2, 'position');
%     leg3_pos = get(hleg3, 'position');
%     legend_positions = sortrows([ leg1_pos; leg2_pos; leg3_pos ], 1);
%     leg_width = max(legend_positions(:, 3));
%     max_height = max(legend_positions(:, 4));
% 
%     set(hleg1, 'units', 'normalized', 'position', [ legend_positions(1, 1) leg1_pos(2) leg_width max_height ]);
%     set(hleg2, 'units', 'normalized', 'position', [ legend_positions(1, 1) leg2_pos(2) leg_width max_height ]);
%     set(hleg3, 'units', 'normalized', 'position', [ legend_positions(1, 1) leg3_pos(2) leg_width max_height ]);
%     
%     % set width of all plots to the same
%     sub1_pos = get(sub{1}, 'position');
%     sub2_pos = get(sub{2}, 'position');
%     sub3_pos = get(sub{3}, 'position');
%     sub_positions = sortrows([ sub1_pos; sub2_pos; sub3_pos ], 3);
%     max_width = max(sub_positions(:, 3));
%     set(sub{1}, 'units', 'normalized', 'position', [ sub1_pos(1:2) legend_positions(1, 1) sub1_pos(4) ]);
%     set(sub{2}, 'units', 'normalized', 'position', [ sub2_pos(1:2) legend_positions(1, 1) sub2_pos(4) ]);
%     set(sub{3}, 'units', 'normalized', 'position', [ sub3_pos(1:2) legend_positions(1, 1) sub3_pos(4) ]);
    
    sub = cellfun(@set_x_axis, sub, 'UniformOutput', false);
    
%     sub{3} = subplot(4, 1, 4);
%     hold on
%     %plot(combined_xmit_times);
%     plot(xmit_profile_number, xmit_ascent_start_time_ori, 'marker', '+');
%     plot(xmit_profile_number, xmit_ascent_end_time, 'marker', 'o');
%     plot(xmit_profile_number, xmit_descent_start_time, 'marker', 'x');
%     plot(xmit_profile_number, xmit_parking_start_time, 'marker', '*');
%     plot(xmit_profile_number, xmit_upcast_start_time, 'marker', '+');
%     plot(xmit_profile_number, xmit_surface_start_time, 'marker', '+');
%     legend('ascentstart', 'ascentend', 'descentstart', 'parkingstart', 'upcaststart', 'surfacestart');
%     box on
%     grid on
%     hold off
    
    
    
    % create main title
%     heading = utils.suplabel(...
%         [ 'Float ' ...
%           num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number) ...
%           ' @ ' ...
%           datestr(datetime('now', 'Format', 'dd-MMM-yyyy HH:mm:SS')) ...
%         ], 't');
%     set(heading, 'FontSize', 18);
    
    % store plot
    
    image_filename = [p_config.PLOT_TIMING_INFORMATION.image_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    figure_filename = [p_config.PLOT_TIMING_INFORMATION.figure_filename ...
        num2str(p_nemo_data{o_float}{1}.FLOAT_IDENTIFICATION.float_serial_number)];
    
    if (p_config.PLOT_TIMING_INFORMATION.save_figure)
        set(gcf,'CreateFcn','set(gcf,''Visible'',''on'')');
        saveas(figure_id, ...
               [directory_fig figure_filename]);
     end
    
    
    print(figure_id, [directory image_filename], image_type);
    
    close(figure_id);

end

boolean = true;

    function current_plot = set_x_axis(current_plot)
        
        % get min and max datevec
        min_datenum = min(datenum(all_profile_data.best_guess_datevec));
        max_datenum = max(datenum(all_profile_data.best_guess_datevec));
        if (min_datenum == max_datenum)
            max_datenum = max_datenum + 1;
        end
        diff_datenum = max_datenum - min_datenum;
        if (diff_datenum > 365)
            x_tick = [min_datenum:365:max_datenum];
        elseif (diff_datenum > 30)
            x_tick = [min_datenum:30:max_datenum];
        elseif (diff_datenum > 5)
            x_tick = [min_datenum:5:max_datenum];
        else
            x_tick = [min_datenum:1:max_datenum];
        end
        if (x_tick(end) < max_datenum)
            x_tick = [ x_tick max_datenum ];
        end
        x_tick_labels = cellstr(datestr(x_tick, 'yyyy-mm-dd'));
        
        if (~isempty(x_tick_labels))
            x_tick_labels{end} = '';
        end
        
        set(current_plot, 'XLimMode', 'manual', ...
            'XLim', [min_datenum, max_datenum], ...
            'XTick', x_tick, ...
            'XTickLabel', x_tick_labels);
        
    end

    function [value, index] = find_nearest_value_to(p_matrix, p_value)
        [value, index] = min(abs(p_matrix - p_value));
        if (length(value) > 1)
            index = min(index);
            value = p_matrix(index);
        end
    end

end

